# -*- coding: utf-8 -*-

##
# Module dependencies

import os.path
import json
import shutil
import requests


##
# Sends request to the Pixaven Image API

def sendRequest(options):
    if 'errorMessage' in options:
        if 'toBuffer' in options:
            return options['errorMessage'], None, None
        else:
            return options['errorMessage'], None


    ##
    # Define empty headers hash wich will be injected
    # along with Requests parameters

    headers = {}


    ##
    # Set up HTTP proxy

    if 'proxy' in options:
        proxy = options['proxy']


    ##
    # Set the response mode to binary
    # and stream the response back
    # when dealing with Binary Responses

    if 'toFile' in options or 'toBuffer' in options:
        options['request']['response'] = {
            'mode': 'binary'
        }

        headers['X-Pixaven-Binary'] = '1'
        stream = True


    ##
    # Use JSON request for fetch() mode

    if 'withFetch' in options:
        endpoint = 'https://api.pixaven.com/1.0/fetch'
        headers['Content-Type'] = 'application/json'


    ##
    # Use Multipart request for upload() mode

    if 'withUpload' in options:
        endpoint = 'https://api.pixaven.com/1.0/upload'

        if type(options['file']) is not str:
            file = options['file'].getvalue()
        else:
            if (os.path.isfile(options['file']) is False):
                error = 'Input file `' + options['file'] + '` does not exist'

                if 'toBuffer' in options:
                    return error, None, None
                else:
                    return error, None

            file = open(options['file'], 'rb')

        files = {
            'file': file
        }


    ##
    # Stringify processing parameters

    data = {
        'data': json.dumps(options['request'])
    }


    ##
    # Initiate POST request to the API

    r = requests.post(
        endpoint,
        auth = (options['key'], ''),
        headers = headers,
        files = files if 'files' in vars() else None,
        proxies = proxy if 'proxy' in vars() else None,
        stream = stream if 'stream' in vars() else False,
        data = data
    )


    ##
    # Parse the response body when dealing with toJSON() requests
    # and return the data to the user

    if 'toJSON' in options:
        try:
            response = r.json()
        except Exception as e:
            error = 'Unable to parse JSON response from the Pixaven Image API'

            if 'toBuffer' in options:
                return error, None, None
            else:
                return error, None

        if response['success'] is False:
            error = response['message']

            if 'toBuffer' in options:
                return error, None, None
            else:
                return error, None

        return None, response


    ##
    # Try to parse JSON data from X-Pixaven-Meta header

    try:
        meta = json.loads(r.headers['X-Pixaven-Meta'])
    except Exception as e:
        error = 'Unable to parse JSON data from X-Pixaven-Meta header'

        if 'toBuffer' in options:
            return error, None, None
        else:
            return error, None


    ##
    # Check whether the API call resulted with a failed response
    # and pass the error message to the user

    if meta['success'] is False:
        error = meta['message']

        if 'toBuffer' in options:
            return error, meta, None
        else:
            return error, meta


    ##
    # Stream incoming binary data to disk
    # when dealing with toFile() requests

    if 'toFile' in options:
        try:
            with open(options['outputPath'], 'wb') as f:
                r.raw.decode_content = True
                shutil.copyfileobj(r.raw, f)
        except Exception as e:
            error = 'Unable to save resulting image at `' + options['outputPath'] + '` location'

            if 'toBuffer' in options:
                return error, meta, None
            else:
                return error, meta

        return None, meta, None


    ##
    # Return raw data to the user
    # when dealing with toBuffer() requests

    if 'toBuffer' in options:
        r.raw.decode_content = True
        return None, meta, r.raw.data


# -*- coding: utf-8 -*-

import os
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup (
    name = 'pixaven',
    version = '1.0.2',
    description = 'Official Python integration for the Pixaven Image API',
    long_description = 'Pixaven is an enterprise-grade image processing SaaS running entirely on Apple® platform. Resize, scale, crop, mask, filter and enhance your images with blazing speed.',
    url = 'https://github.com/pixaven/pixaven-python',
    download_url = 'https://github.com/pixaven/pixaven-python/archive/1.0.0.tar.gz',
    author = 'Pixaven UG',
    author_email = 'support@pixaven.com',
    license = 'MIT',
    keywords = 'pixaven image processing resizing resizer cropping scaling masking watermarking filtering thumbnails pic picture photo face face detection visual watermark filter crop mask resize resizer thumbs thumbnail thumbnails jpg jpeg png gif svg bmp psd tiff',

    packages = [
        'pixaven'
    ],

    install_requires = [
        'requests'
    ],

    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
    ]
)


# -*- coding: utf-8 -*-

##
# Module dependencies

import json
import requests

from . import client


##
# Pixaven class

class pixaven(object):
    def __init__(self, key = None):
        self.options = {
            'key': key,
            'request': {}
        }

        if key is None or key is not key or type(key) is not str:
            self.options['errorMessage'] = 'Pixaven constructor requires a valid API Key'


    ##
    # Uploads an image for processing
    #
    # @param {String|Buffer} file
    # @returns {Pixaven}

    def upload(self, file = ''):
        if 'withFetch' in self.options:
            self.options['errorMessage'] = 'Pixaven only accepts one file input method per call: upload(String|Buffer) or fetch(String)'

        if file is None or file is not file:
            self.options['errorMessage'] = 'Pixaven upload(String|Buffer) method requires file path or a Buffer passed as an argument'
        else:
            self.options['withUpload'] = True
            self.options['file'] = file

        return self


    ##
    # Provides a URL of the image for processing
    #
    # @param {String} url
    # @returns {Pixaven}

    def fetch(self, url = ''):
        if 'withUpload' in self.options:
            self.options['errorMessage'] = 'Pixaven only accepts one file input method per call: upload(String|Buffer) or fetch(String)'

        if file is None or file is not file or type(url) is not str:
            self.options['errorMessage'] = 'Pixaven fetch(String) method requires a valid file URL passed as an argument'
        else:
            self.options['withFetch'] = True
            self.options['url'] = file

        return self


    ##
    # Sets a proxy for HTTP requests
    #
    # @param {Dict} proxy
    # @returns {Pixaven}

    def setProxy(self, proxy = {}):
        if proxy is not None and type(proxy) is dict:
            self.options['proxy'] = proxy

        return self


    ##
    # Flips an image
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def flip(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['flip'] = data

        return self


    ##
    # Resizes an image
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def resize(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['resize'] = data

        return self


    ##
    # Scales an image
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def scale(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['scale'] = data

        return self


    ##
    # Applies a watermark
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def watermark(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['watermark'] = data

        return self


    ##
    # Applies an elliptical mask
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def mask(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['mask'] = data

        return self


    ##
    # Applies a filter
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def filter(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['filter'] = data

        return self


    ##
    # Adjusts visual parameters
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def adjust(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['adjust'] = data

        return self


    ##
    # Automatically enhances an image
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def auto(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['auto'] = data

        return self


    ##
    # Applies a border to an image
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def border(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['border'] = data

        return self


    ##
    # Pads the image
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def padding(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['padding'] = data

        return self


    ##
    # Stores processed image externally
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def store(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['store'] = data

        return self


    ##
    # Sets output format and encoding
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def output(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['output'] = data

        return self


    ##
    # Sets a Webhook as a reponse delivery method
    #
    # @param {Dict} data
    # @returns {Pixaven}

    def webhook(self, data = {}):
        if data is not None and type(data) is dict:
            self.options['request']['webhook'] = data

        return self


    ##
    # Sends a standard request to the API
    # and returns a JSON response
    #
    # @returns {String, Dict}

    def toJSON(self):
        if 'toFile' in self.options or 'toBuffer' in self.options:
            self.options['errorMessage'] = 'Pixaven only accepts one response method per call: toJSON(), toFile(String) or toBuffer()'

        if 'withUpload' not in self.options and 'withFetch' not in self.options:
            self.options['errorMessage'] = 'No file input has been specified with either upload(String|Buffer) or fetch(String) method'

        self.options['toJSON'] = True

        return client.sendRequest(self.options)


    ##
    # Instructs the API to use a Binary Response
    # and streams the response to disk
    #
    # @param {String} path
    # @returns {String, Dict}

    def toFile(self, path = ''):
        if path is None or path is not path or type(path) is not str:
            self.options['errorMessage'] = 'Pixaven toFile(String) method requires a valid output file path as a parameter'

        if 'toJSON' in self.options or 'toBuffer' in self.options:
            self.options['errorMessage'] = 'Pixaven only accepts one response method per call: toJSON(), toFile(String) or toBuffer()'

        if 'webhook' in self.options['request']:
            self.options['errorMessage'] = 'Binary responses with toFile(String) method are not supported when using Webhooks'

        if 'store' in self.options['request']:
            self.options['errorMessage'] = 'Binary responses with toFile(String) method are not supported when using External Storage'

        if 'withUpload' not in self.options and 'withFetch' not in self.options:
            self.options['errorMessage'] = 'No file input has been specified with either upload(String|Buffer) or fetch(String) method'

        self.options['toFile'] = True
        self.options['outputPath'] = path

        return client.sendRequest(self.options)


    ##
    # Instructs the API to use a Binary Response
    # and returns a buffer to the user
    #
    # @param {Array} data
    # @returns {String, Dict, Buffer}

    def toBuffer(self):
        if 'toJSON' in self.options or 'toFile' in self.options:
            self.options['errorMessage'] = 'Pixaven only accepts one response method per call: toJSON(), toFile(String) or toBuffer()'

        if 'webhook' in self.options['request']:
            self.options['errorMessage'] = 'Binary responses with toBuffer() method are not supported when using Webhooks'

        if 'store' in self.options['request']:
            self.options['errorMessage'] = 'Binary responses with toBuffer() method are not supported when using External Storage'

        if 'withUpload' not in self.options and 'withFetch' not in self.options:
            self.options['errorMessage'] = 'No file input has been specified with either upload(String|Buffer) or fetch(String) method'

        self.options['toBuffer'] = True

        return client.sendRequest(self.options)